import{a as t,j as o,t as f,r as n,w as a,g as b,n as u,e as v}from"./index-BtH6urND.js";var g=o('<iframe class="html-frame svelte-1cbd4lv" title="HTML preview"></iframe>');function y(l,e){let d=t(e,"html",3,""),r=t(e,"css",3,void 0),i=t(e,"js",3,void 0),m=t(e,"height",3,"400px"),h=t(e,"sandbox",3,"allow-scripts");const c=u(()=>`<!DOCTYPE html><html><head><style>${r()??""}</style></head><body>${d()??""}<script>${i()??""}<\/script></body></html>`);var s=g();f(()=>{n(s,`height: ${m()};`),a(s,"sandbox",h()),a(s,"srcdoc",b(c))}),v(l,s)}export{y as default};
