import{p as B,a as u,u as E,c as T,s as F,j as p,z as G,i as b,g as a,l as d,y,t as g,q as j,k as q,e as o,A as H,n as w,h as I,J as K,d as L,f as O,D as P}from"./index-BtH6urND.js";import Q from"./Markdown-BDB-IX6n.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var S=p('<span class="icon"> </span>'),U=p("<button><!> </button>"),V=p('<div class="empty">No content.</div>'),W=p('<div><div class="tab-list svelte-1f2pv7a"></div> <div class="tab-panel svelte-1f2pv7a"><!></div></div>');function ta(z,n){B(n,!0);let h=u(n,"tabs",19,()=>[]),A=u(n,"defaultTab",3,0),C=u(n,"variant",3,"line"),i=F(0);E(()=>{T(i,A())});const v=w(()=>h()[a(i)]);var f=W(),k=d(f);G(k,21,h,H,(e,t,l)=>{var r=U();r.__click=()=>T(i,l,!0);var c=d(r);{var s=m=>{var x=S(),R=d(x);g(()=>q(R,a(t).icon)),o(m,x)};b(c,m=>{a(t).icon&&m(s)})}var _=y(c);g(()=>{j(r,1,`tab ${l===a(i)?"active":""}`,"svelte-1f2pv7a"),r.disabled=a(t).disabled,q(_,` ${a(t).label??""}`)}),o(e,r)});var D=y(k,2),J=d(D);{var M=e=>{{let t=w(()=>a(v).props??{});K(e,{get component(){return a(v).component},get props(){return a(t)}})}},N=e=>{var t=L(),l=O(t);{var r=s=>{Q(s,{get content(){return a(v).content},padded:!1})},c=s=>{var _=V();o(s,_)};b(l,s=>{a(v)?.content?s(r):s(c,!1)},!0)}o(e,t)};b(J,e=>{a(v)?.component?e(M):e(N,!1)})}g(()=>j(f,1,`tabs ${C()}`,"svelte-1f2pv7a")),o(z,f),I()}P(["click"]);export{ta as default};
