.select-option.svelte-hx6wcr{padding:1.25rem}h3.svelte-hx6wcr{margin:0}.header.svelte-hx6wcr p:where(.svelte-hx6wcr){margin:.25rem 0 0;color:#6b7280}.search.svelte-hx6wcr{margin-top:.75rem;width:100%;padding:.4rem .6rem;border-radius:.375rem;border:1px solid #d1d5db}.options.svelte-hx6wcr{margin-top:.75rem;display:grid;gap:.75rem}.options.grid.svelte-hx6wcr{grid-template-columns:repeat(auto-fit,minmax(160px,1fr))}.options.cards.svelte-hx6wcr{grid-template-columns:repeat(auto-fit,minmax(200px,1fr))}.option.svelte-hx6wcr{padding:.75rem;text-align:left;border:1px solid #e5e7eb;border-radius:.5rem;background:#fff;cursor:pointer}.option.active.svelte-hx6wcr{border-color:#2563eb;box-shadow:0 0 0 3px #2563eb1a}.option-desc.svelte-hx6wcr{font-size:.75rem;color:#64748b}.actions.svelte-hx6wcr{display:flex;justify-content:flex-end;gap:.75rem;margin-top:1rem;align-items:center}.error.svelte-hx6wcr{margin-right:auto;font-size:.75rem;color:#dc2626}.btn-submit.svelte-hx6wcr{background:#2563eb;color:#fff;border:none;padding:.4rem .75rem;border-radius:.375rem}.btn-cancel.svelte-hx6wcr{background:#fff;border:1px solid #d1d5db;padding:.4rem .75rem;border-radius:.375rem}
