.confirm.svelte-1dszvfa{padding:1.25rem;border-radius:var(--radius-2xl, 18px);border:1px solid var(--color-border, #f0ebe4);background:var(--color-card-bg, #fcfaf7);box-shadow:var(--shadow-card, 0 12px 32px rgba(17, 17, 17, .06))}.confirm.warning.svelte-1dszvfa{border-color:var(--color-border, #f0ebe4)}.confirm.danger.svelte-1dszvfa{border-color:var(--color-error-border, #f5dddd)}.confirm.success.svelte-1dszvfa{border-color:var(--color-border, #f0ebe4)}h3.svelte-1dszvfa{margin:0;font-size:1.1rem;color:var(--color-text, #1f1f1f)}.confirm-body.svelte-1dszvfa{margin-top:.75rem;color:var(--color-text-secondary, #3c3a36)}.confirm-actions.svelte-1dszvfa{display:flex;justify-content:flex-end;gap:.75rem;margin-top:1rem}.btn-cancel.svelte-1dszvfa{border:1px solid var(--color-border, #f0ebe4);background:var(--color-btn-ghost-bg, #f2eee8);padding:.4rem .75rem;border-radius:var(--radius-md, 10px);cursor:pointer}.btn-confirm.svelte-1dszvfa{border:none;background:var(--color-btn-primary-gradient, linear-gradient(135deg, #31a6a0, #1a7c75));color:#fff;padding:.4rem .75rem;border-radius:var(--radius-md, 10px);cursor:pointer;box-shadow:0 0 0 0 var(--color-btn-primary-shadow, rgba(49, 166, 160, .3))}
