import{p as je,a as C,j as v,i,l as o,y as d,g as e,n as xe,z as J,t as p,w as ne,e as r,k as u,d as K,f as N,A as L,J as ie,h as we,r as ke}from"./index-BtH6urND.js";import O from"./Markdown-BDB-IX6n.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var Ce=v('<h1 class="svelte-djhbyc"> </h1>'),Re=v('<p class="subtitle svelte-djhbyc"> </p>'),ze=v("<span> </span>"),Ae=v("<span> </span>"),Be=v("<span> </span>"),Ee=v('<div class="meta svelte-djhbyc"><!> <!> <!></div>'),Je=v('<li class="svelte-djhbyc"><a class="svelte-djhbyc"> </a></li>'),Le=v('<nav class="toc svelte-djhbyc"><h2 class="svelte-djhbyc">Contents</h2> <ul class="svelte-djhbyc"></ul></nav>'),Me=v('<h2 class="svelte-djhbyc"> </h2>'),Te=v('<figcaption class="svelte-djhbyc"> </figcaption>'),qe=v('<figure class="component-figure svelte-djhbyc"><!> <!></figure>'),De=v('<h3 class="svelte-djhbyc"> </h3>'),Fe=v('<figcaption class="svelte-djhbyc"> </figcaption>'),Ge=v('<figure class="component-figure svelte-djhbyc"><!> <!></figure>'),He=v('<section class="subsection svelte-djhbyc"><!> <!> <!></section>'),Ie=v('<section class="svelte-djhbyc"><!> <!> <!> <!></section>'),Ke=v('<footer class="report-footer svelte-djhbyc"><!></footer>'),Ne=v('<article class="artifact-report svelte-djhbyc"><header class="report-header svelte-djhbyc"><!> <!> <!></header> <!> <div class="report-body svelte-djhbyc"></div> <!></article>');function Ve(le,j){je(j,!0);let P=C(j,"title",3,""),Q=C(j,"subtitle",3,""),g=C(j,"metadata",19,()=>({})),S=C(j,"toc",3,!1),U=C(j,"sections",19,()=>[]),V=C(j,"footer",3,"");function W(a){return a.toLowerCase().replace(/\s+/g,"-").replace(/[^\w-]/g,"")}function X(a,t=0){const s=[];for(const c of a)c.title&&s.push({title:c.title,id:c.id||W(c.title),level:t}),c.subsections&&s.push(...X(c.subsections,t+1));return s}const Y=xe(()=>S()?X(U()):[]);var Z=Ne(),$=o(Z),ee=o($);{var de=a=>{var t=Ce(),s=o(t);p(()=>u(s,P())),r(a,t)};i(ee,a=>{P()&&a(de)})}var te=d(ee,2);{var ce=a=>{var t=Re(),s=o(t);p(()=>u(s,Q())),r(a,t)};i(te,a=>{Q()&&a(ce)})}var _e=d(te,2);{var fe=a=>{var t=Ee(),s=o(t);{var c=l=>{var h=ze(),w=o(h);p(()=>u(w,`By ${g().author??""}`)),r(l,h)};i(s,l=>{g().author&&l(c)})}var b=d(s,2);{var m=l=>{var h=Ae(),w=o(h);p(()=>u(w,g().date)),r(l,h)};i(b,l=>{g().date&&l(m)})}var k=d(b,2);{var x=l=>{var h=Be(),w=o(h);p(()=>u(w,`v${g().version??""}`)),r(l,h)};i(k,l=>{g().version&&l(x)})}r(a,t)};i(_e,a=>{(g().author||g().date)&&a(fe)})}var ae=d($,2);{var pe=a=>{var t=Le(),s=d(o(t),2);J(s,21,()=>e(Y),L,(c,b)=>{var m=Je();let k;var x=o(m),l=o(x);p(()=>{k=ke(m,"",k,{"margin-left":`${e(b).level*1}rem`}),ne(x,"href",`#${e(b).id}`),u(l,e(b).title)}),r(c,m)}),r(a,t)};i(ae,a=>{S()&&e(Y).length>0&&a(pe)})}var re=d(ae,2);J(re,21,U,L,(a,t)=>{var s=Ie(),c=o(s);{var b=n=>{var y=Me(),R=o(y);p(()=>u(R,e(t).title)),r(n,y)};i(c,n=>{e(t).title&&n(b)})}var m=d(c,2);{var k=n=>{O(n,{get content(){return e(t).content},padded:!1})};i(m,n=>{e(t).content&&n(k)})}var x=d(m,2);{var l=n=>{var y=K(),R=N(y);J(R,17,()=>e(t).components,L,(D,_)=>{var z=qe(),A=o(z);ie(A,{get component(){return e(_).component},get props(){return e(_).props}});var F=d(A,2);{var M=B=>{var T=Te(),G=o(T);p(()=>u(G,e(_).caption)),r(B,T)};i(F,B=>{e(_).caption&&B(M)})}r(D,z)}),r(n,y)};i(x,n=>{e(t).components&&n(l)})}var h=d(x,2);{var w=n=>{var y=K(),R=N(y);J(R,17,()=>e(t).subsections,L,(D,_)=>{var z=He(),A=o(z);{var F=f=>{var E=De(),H=o(E);p(()=>u(H,e(_).title)),r(f,E)};i(A,f=>{e(_).title&&f(F)})}var M=d(A,2);{var B=f=>{O(f,{get content(){return e(_).content},padded:!1})};i(M,f=>{e(_).content&&f(B)})}var T=d(M,2);{var G=f=>{var E=K(),H=N(E);J(H,17,()=>e(_).components,L,(ge,q)=>{var oe=Ge(),se=o(oe);ie(se,{get component(){return e(q).component},get props(){return e(q).props}});var be=d(se,2);{var me=I=>{var ve=Fe(),ye=o(ve);p(()=>u(ye,e(q).caption)),r(I,ve)};i(be,I=>{e(q).caption&&I(me)})}r(ge,oe)}),r(f,E)};i(T,f=>{e(_).components&&f(G)})}r(D,z)}),r(n,y)};i(h,n=>{e(t).subsections&&n(w)})}p(n=>ne(s,"id",n),[()=>e(t).id||(e(t).title?W(e(t).title):void 0)]),r(a,s)});var he=d(re,2);{var ue=a=>{var t=Ke(),s=o(t);O(s,{get content(){return V()},padded:!1}),r(a,t)};i(he,a=>{V()&&a(ue)})}r(le,Z),we()}export{Ve as default};
