import{p as B,a as t,j as h,y as l,l as e,i as E,t as F,q as G,k as d,e as _,h as I,D as J}from"./index-BtH6urND.js";import b from"./Markdown-BDB-IX6n.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var K=h("<details><summary>Details</summary> <!></details>"),N=h('<div><div class="confirm-header"><h3 class="svelte-1dszvfa"> </h3></div> <div class="confirm-body svelte-1dszvfa"><!> <!></div> <div class="confirm-actions svelte-1dszvfa"><button class="btn-cancel svelte-1dszvfa"> </button> <button class="btn-confirm svelte-1dszvfa"> </button></div></div>');function U(g,a){B(a,!0);let k=t(a,"title",3,"Confirm"),z=t(a,"message",3,""),C=t(a,"confirmLabel",3,"Confirm"),x=t(a,"cancelLabel",3,"Cancel"),y=t(a,"variant",3,"info"),r=t(a,"details",3,void 0),o=t(a,"onResult",3,void 0);function L(){o()?.({confirmed:!0})}function p(){o()?.({confirmed:!1})}var i=N(),v=e(i),w=e(v),D=e(w),c=l(v,2),f=e(c);b(f,{get content(){return z()},allowHtml:!1,padded:!1});var H=l(f,2);{var R=s=>{var u=K(),A=l(e(u),2);b(A,{get content(){return r()},allowHtml:!1,padded:!1}),_(s,u)};E(H,s=>{r()&&s(R)})}var j=l(c,2),n=e(j);n.__click=p;var q=e(n),m=l(n,2);m.__click=L;var M=e(m);F(()=>{G(i,1,`confirm ${y()}`,"svelte-1dszvfa"),d(D,k()),d(q,x()),d(M,C())}),_(g,i),I()}J(["click"]);export{U as default};
